
# q-expansion of newform 4800.2.a.g, downloaded from the LMFDB on 03 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4800
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [0], [-3], [-4], [7], [4], [1], [-8], [0], [-3], [-2], [-6], [11], [6], [-6], [6], [1], [15], [6], [2], [-8], [-2], [-16], [13], [2], [-16], [18], [-17], [-12], [8], [-10], [-14], [-4], [-6], [-13], [-7], [-7], [-6], [-16], [18], [-13], [-20], [-13], [-26], [-17], [-19], [-15], [10], [-9], [18], [10], [5], [-24], [-24], [26], [20], [20], [3], [8], [-23], [-14], [17], [-18], [-17], [12], [24], [5], [-14], [14], [-16], [-10], [15], [25], [-17], [-8], [-6], [-27], [-38], [29], [6], [2], [-8], [23], [13], [-20], [-8], [26], [-10], [4], [-12], [2], [-25], [-24], [-11], [26], [-10], [24], [1], [3], [8], [6], [-36], [-32], [-23], [-1], [-30], [-42], [-8], [19], [32], [26], [42], [1], [-35], [32], [-12], [-28], [-10], [10], [26], [-18], [12], [24], [20], [20], [35], [8], [-11], [-22], [20], [12], [8], [47], [12], [7], [-46], [-11], [-40], [14], [21], [-6], [37], [18], [18], [12], [21], [8], [-28], [-44], [5], [-36], [7], [-38], [-36], [58], [-53], [36], [27], [50], [40], [-24], [-8], [-34], [30], [-52], [13], [30], [-11], [42], [40], [-31], [-18], [9], [-1], [-12], [21], [0], [-19], [13], [-59], [-16], [-13], [-6], [-18], [50], [27], [1], [1], [12], [65], [34], [-9], [-8], [-8], [-50], [-42], [-30], [-42], [68], [38], [48], [-30], [-38], [-12], [48], [-65], [20], [8], [-25], [7], [30], [1], [36], [-34], [-26], [-35], [0], [-18], [-20], [-5], [35], [-6], [-20], [62], [1], [-4], [-54], [-29], [0], [-45], [-16], [-37], [-34], [-16], [-32], [-54], [2], [2], [-42], [52], [47], [7], [57], [-58], [-70], [47], [-56], [3], [26], [30], [-28], [48], [-19], [-24], [-36], [-5], [-8], [-18], [46], [-7], [0], [70], [-11], [18], [44], [-38], [12], [20], [50], [-27], [-44], [38], [57], [-51], [29], [-10], [-19], [27], [-60], [42], [-16], [-24], [-27], [45], [-2], [25], [-50], [-8], [34], [-54], [-8], [72], [52], [-47], [46], [-67], [46], [0], [-12], [30], [20], [65], [-66], [-4], [37], [-40], [47], [-6], [-66], [32], [48], [52], [-61], [20], [1], [-40], [-84], [73], [52], [44], [23], [-6], [9], [70], [50], [-79], [-31], [0], [30], [35], [26], [-31], [16], [-19], [92], [-31], [-54], [17], [56], [-31], [-18], [0], [-69], [12], [-14], [31], [29], [-18], [-48], [-75], [70], [80], [19], [-3], [90], [-14], [12], [-48], [-16], [-55], [34], [-52], [4], [72], [-81], [48], [24], [-17], [-36], [-25], [68], [-56], [0], [-37], [78], [-58], [7], [76], [-23], [46], [-36], [-80], [-22], [-84], [18], [17], [-10], [-48], [12], [-15], [-66], [-26], [-29], [-98], [-59], [-23], [-26], [55], [8], [-2], [-6], [-85], [46], [46], [8], [19], [66], [69], [40], [-57], [92], [-78], [73], [-39], [-100], [-18], [43], [-18], [-34], [-20], [73], [-22], [6], [-26], [102], [78], [6], [-5], [48]]
